#include <stdio.h>
#include <stdlib.h>
#include "af2_par.h"

void csub_getenv__(name,trans,name_length,trans_length,status)
/*----------------------------------------------------------------------------*/
/*                                                                            */
/*    Name:                                                                   */
/*       csub_getenv_                                                         */
/*                                                                            */
/*    Invocation:                                                             */
/*       call csub_getenv(name,trans,name_length,trans_length,status)         */
/*                                                                            */
/*    Description:                                                            */
/*       Get the translation of an environment variable.  This is meant to    */
/*       be called from a FORTRAN routine.                                    */
/*                                                                            */
/*    Arguments: (">" Input, "<" Output, "W" Workspace, "!"  Changed)         */
/*       name:  Char* (>)                                                     */
/*          The FORTRAN character string with the name of the environment     */
/*          variable to be chased up.                                         */
/*       trans:  Char* (<)                                                    */
/*          The returned FORTRAN character string with the translation        */
/*       name_length:  Int* (>)                                               */
/*          The length of the name string                                     */
/*       trans_length:  Int* (>)                                              */
/*          The length of the translation string                              */
/*       status:  Int* (!)                                                    */
/*          The global status variable                                        */
/*                                                                            */
/*    Language:                                                               */
/*       ANSI C                                                               */
/*                                                                            */
/*    Non-standard language extensions:                                       */
/*       None                                                                 */
/*                                                                            */
/*    Algorithm:                                                              */
/*       -- Find the last non-blank character in the FORTRAN string and       */
/*          allocate workspace to copy the non-blank bit into.                */
/*       -- Copy the FORTRAN string over and use the intrinsic getenv         */
/*          function to get the translation.                                  */
/*       -- Copy the translation over to the FORTRAN translation string,      */
/*          padding the extra out with blanks                                 */
/*       -- Free the workspace and exit                                       */
/*                                                                            */
/*    Authors:                                                                */
/*       (JRL): James R. Lewis -- RGO, Cambridge                              */
/*                                                                            */
/*    History:                                                                */
/*       01 Dec 1995 (JRL):                                                   */
/*          Original version                                                  */
/*                                                                            */
/*----------------------------------------------------------------------------*/

/* Parameters */
char *const name;
char *const trans;
int *const name_length;
int *const trans_length;
int *const status;

{

      /* Local variables */
      int i;
      char *ptr,*ptr2;

      /* Inherited status */
      if (*status != AF2_OK) return;

      /* Locate the last non-blank character in input FORTRAN string */
      /* Don't forget the the first element is actually 0 not 1 */
      for (i = *name_length - 1; i >= 0 && name[i] == ' '; i--)
         ;

      /* Get some space for a copy of the input string (plus one extra */
      /* for the EOS marker) */
      ptr = (char *)malloc((size_t)(i+2));

      /* If the null pointer came up, then signal an error */
      if (ptr == 0) {
         fprintf(stderr,"Couldn't allocate %d bytes in GETENV\n",i+2);
         *status = AF2_ERR;
         return;
      }

      /* Otherwise copy the string over...*/
      ptr[i+1] = '\0';
      for (; i >= 0; i--)
         ptr[i] = name[i];

      /* Ok, now get the environment name */
      ptr2 = getenv(ptr);

      /* If the pointer came out null, then signal an error */
      if (ptr2 == 0) {
         fprintf(stderr,"No translation for %s \n",ptr);
         *status = AF2_ERR;
         return;
      }

      /* Now copy the answer back over to a FORTRAN string... */
      for (i = 0; i < *trans_length && ptr2[i] != '\0'; i++)
         trans[i] = ptr2[i];

      /* Pad out the FORTRAN string with blanks */
      for ( ; i < *trans_length; i++)
         trans[i] = ' ';

      /* Free workspace */
      free((void *)ptr);
}
