      subroutine do_fibre_check(npivots,piv1,but1,fibdims,curfib, 
     :   piv,fib,fibrelen2,maxnonrad,fibstatus,offender,fstatus)
C+------------------------------------------------------------------------------
C
C     Name:   
C        do_fibre_check
C
C     Invocation:
C        call do_fibre_check(npivots,piv1,but1,fibdims,curfib,
C           piv,fib,fibrelen2,maxnonrad,fibstatus,offender,fstatus)
C
C     Description:
C        A single fibre is checked to see if it collides with any of the
C        other fibres.
C
C     Arguments: (">" Input, "<" Output, "W" Workspace, "!"  Changed)
C        npivots:  Integer (>)
C           The number of pivots
C        piv1:  Real array (>)
C           The position of the pivot of the fibre you wish to check
C        but1:  Real array (>)
C           The position of the button of the fibre you wish to check
C        fibdims:  Integer array (>)
C           The dimensions of the fibres: 1 -- button length, 2 -- button
C           width, 3 -- tube width
C        curfib:  Integer array (>)
C           The pivot number of the fibre in question.
C        piv:  Integer array (>)
C           The coordinates of the all of the pivots.
C        fib:  Integer array (>)
C           The coordinates of all of the fibre buttons
C        fibrelen2:  Real (>)
C           The square of the length of a fibre
C        maxnonrad:  Real (>)
C           The maximum non-radial angle a fibre is allowed to subtend.
C        fibstatus:  Integer array (>)
C           Array of values which gives the fibre type for each pivot
C           (i.e. fiducial, object, disabled, etc).
C        offender:  Integer (<)
C           If there is a collision, this is the pivot number of the 
C           offending fibre.
C        fstatus:  Integer (<)
C           Status variable which tells whether there has been a collision
C           or not and if so, what type.  This is assigned values from the
C           include file 'af2_fchk.inc'.
C
C     Language:
C        FORTRAN 77
C
C     Non-standard language extensions:
C        include, implicit none, do while, variable names > 6 characters
C
C     Logical i/o units used:
C        none
C
C     External routines called:
C        Library subaf2_configure:
C           fibre_check
C
C        Library slalib:
C           sla_range
C 
C     Algorithm:
C        -- Start out by assuming there is no collision
C        -- Look at the next highest numbered pivot to the fibre in 
C           question.  If there is a collision, the exit.
C        -- Look at the next lowest numbered pivot and do the same check
C        -- Keep alternating between the next highest and next lowest numbered
C           fibres and work your way around the circle of pivots.  Exit
C           the loop as soon as a collision is found.
C        -- If no collision is found, then exit after checking all of the
C           fibres once.
C
C     Authors:
C        (JRL): James R. Lewis -- RGO, Cambridge
C
C     History:
C        01 Dec 1995 (JRL):
C           Original version
C        13 May 1996 (JRL)
C           Added test to make sure atan2 doesn't ever get a vector of 
C           length zero.
C
C-------------------------------------------------------------------------------
      implicit none
C
C     Global parameters
C
      include 'AF2_FCHK.INC'
      include 'AF2_FSTA.INC'
C
C     Parameters
C
      integer npivots
      integer fibdims(3)
      integer curfib
      integer piv(2,npivots)
      integer fib(2,npivots)
      integer fstatus
      real piv1(2)
      real but1(2)
      real fibrelen2
      real maxnonrad
      integer fibstatus(npivots)
      integer offender
C
C     Local variables
C
      integer ichk1,ichk2,ncheck
      real piv2(2),but2(2),dist2,dx,dy,pivang,theta
C
C     Functions
C
      real sla_range
C
C     Initialise the fibre status and the number of the offender (i.e.
C     the fibre which collides with the current fibre
C
      fstatus = fchk_ok
      offender = 0
C
C     Check non-radial angle.  This is the angular deviation from the
C     parked position angle.  Make sure the calculation gives a result
C     between +/- pi.
C   
      dx = piv1(1) - but1(1)
      dy = piv1(2) - but1(2)
      pivang = atan2(piv1(2),piv1(1))
      if ((dx**2 + dy**2) .gt. 0.0) then
         theta = atan2(dy,dx) - pivang
      else
         theta = -pivang
      end if
      theta = sla_range(theta)
      if (abs(theta) .gt. maxnonrad) then
         fstatus = fchk_maxang
         go to 500
      end if
C 
C     Check distance to pivot.  The distance of the object to the pivot can't
C     be longer than the length of the fibre.  (Compare the squares of the
C     distances to save a bit of time).
C
      dist2 = dx**2 + dy**2
      if (dist2 .gt. fibrelen2) then
         fstatus = fchk_reach
         go to 500
      end if
C
C     Start with the current fibre and check the nearest unchecked
C     fibre on either side until you either have a collision or until
C     all fibres have been checked.  Don't bother checking missing fibres.
C
      ichk1 = curfib
      ichk2 = curfib
      ncheck = 0
      do while (ncheck .lt. npivots-1)
C
C        Increment in the clockwise direction.  If the fibre number is greater
C        then the total number of pivots, then subtract the number of pivots
C        so that things work circularly
C
         ichk1 = ichk1 + 1
         if (ichk1 .gt. npivots) ichk1 = ichk1 - npivots
C
C        If this is not a missing fibre, then check it against the
C        fibre in question
C
         if (fibstatus(ichk1) .ne. fsta_miss) then
            piv2(1) = piv(1,ichk1)
            piv2(2) = piv(2,ichk1)
            but2(1) = fib(1,ichk1)
            but2(2) = fib(2,ichk1)
            call fibre_check(piv1,but1,piv2,but2,fibdims,fstatus)
C
C           If there is a collision, then escape with the number of the
C           offending fibre
C
            if (fstatus .ne. fchk_ok) then
               offender = ichk1
               go to 500
            end if
         end if
C
C        Increment ncheck even though you haven't looked at missing fibres.
C        This is because the do while loop we're in compares this with the
C        total number of pivots.
C
         ncheck = ncheck + 1
C
C        Now look at the next nearest fibre in the anti-clockwise direction.
C        If the number goes less than one, then add in the number of pivots
C        so that this works circularly
C
         ichk2 = ichk2 - 1
         if (ichk2 .lt. 1) ichk2 = ichk2 + npivots
C
C        Since this algorithm works the circle of pivots in alternating
C        directions there is a chance that you've just checked this fibre.
C        In case you have, then skip it.
C
         if (ichk2 .ne. ichk1) then
C
C           Again skip missing fibres
C
            if (fibstatus(ichk2) .ne. fsta_miss) then
               piv2(1) = piv(1,ichk2)
               piv2(2) = piv(2,ichk2)
               but2(1) = fib(1,ichk2)
               but2(2) = fib(2,ichk2)
               call fibre_check(piv1,but1,piv2,but2,fibdims,fstatus)
               if (fstatus .ne. fchk_ok) then
                  offender = ichk2
                  go to 500
               end if
            end if
            ncheck = ncheck + 1
         end if
C
C     End of while loop
C
      end do
C
C     Exit
C
  500 continue
      end
