      subroutine edit(nobj,npivots,nsky,object,object_ref,otype,
     :   plog,weight,n_obj_left,skypos,sky_ref,n_sky_left,slog,fib,
     :   piv,park,fibstatus,allocated,label,piv_r,maxnonrad,fibrelen2,
     :   fibdims,field_size,pgrad,pscale,status)
C+------------------------------------------------------------------------------
C
C     Name:   
C        edit
C
C     Invocation:
C        call edit(nobj,npivots,nsky,object,object_ref,otype,
C           plog,weight,n_obj_left,skypos,sky_ref,n_sky_left,slog,fib,
C           piv,park,fibstatus,allocated,label,piv_r,maxnonrad,fibrelen2,
C           fibdims,field_size,pgrad,pscale,status)
C
C     Description:
C        The incoming configuration is edited interactively.
C
C     Arguments: (">" Input, "<" Output, "W" Workspace, "!"  Changed)
C        nobj:  Integer (>)
C           The total number of objects
C        npivots:  Integer (>)
C           The total number of pivots
C        nsky:  Integer (>)
C           The total number of sky patches
C        object:  Real array (>)
C           The x,y coordinates of the objects (program+fiducial)
C        object_ref:  Integer array (!)
C           The list of reference numbers of objects which have not 
C           been allocated to a fibre.
C        otype:  Integer array (>)
C           The object type (i.e. whether it's a program or sky)
C        plog:  Character array (>)
C           The log of information on each of the objects
C        weight:  Real array (>)
C           The weight assigned to each object
C        n_obj_left:  Integer (!)
C           The number of objects which have not yet been allocated.
C        skypos:  Real array (>)
C           The x,y coordinates of the sky patches
C        sky_ref:  Integer array (!)
C           The list of reference numbers of the sky patches which have
C           not been allocated to a fibre
C        n_sky_left:  Integer (!)
C           The number of sky patches which have not yet been allocated
C        slog:  Character array (>)
C           The log of information on each of the sky patches
C        fib:  Integer array (!)
C           The current x,y position of each fibre button
C        piv:  Integer array (>)
C           The x,y position of each of the pivots
C        park:  Integer array (>)
C           The park position for each of the fibres
C        fibstatus:  Integer array (>)
C           The fibre status (i.e. whether it's useable, disabled etc.)
C        allocated:  Integer array (>)
C           The reference number of the object which has been allocated
C           to a particular fibre.
C        label:  Character (>)
C           A character string specifying a field name label
C        piv_r:  Real (>)
C           The radius of the pivot circle
C        maxnonrad: Real (>)
C           The maximum non-radial angle each fibre is allowed to subtend
C        fibrelen2:  Real (>)
C           The square of length of a fibre
C        fibdims:  Integer array (>)
C           The dimensions of the fibres: 1 -- button length, 2 -- button
C           width, 3 -- tube width
C        field_size:  Real (>)
C           The radius of the accessable field
C        pgrad:  Real (>)
C           The maximum radius of the field to be displayed
C        pscale:  Real array (>)
C           Plot scaling factors: 1 -- the linear scale, 2 -- the text height
C        status:  Integer (!)
C           The global status variable
C
C     Language:
C        FORTRAN 77
C
C     Non-standard language extensions:
C        include, implicit none, do while, variable names > 6 characters
C
C     Logical i/o units used:
C        none
C
C     External routines called:
C        Library subaf2_configure:
C           add_to_list
C           calc_rects
C           count_pl
C           cursor
C           erase
C           findfib
C           findobj
C           graphmsg
C           init_undo
C           movefibre
C           normfibre
C           objinfo
C           objover
C           plotit
C           remove_from_list
C           restore
C           sumweights
C 
C        Library chr:
C           chr_putc
C           chr_puti
C           chr_putr
C 
C        Library pgplot:
C           pgask
C           pgbeg
C           pgend
C           pgpage
C           pgqinf
C           pgsci
C           pgvport
C           pgwnad
C 
C     Algorithm:
C        -- Undo and restore buffers are initialised.
C        -- Keystokes are accepted until we are requested to exit
C        -- On exit the restore buffer is freed
C
C     Authors:
C        (JRL): James R. Lewis -- RGO, Cambridge
C
C     History:
C        01 Dec 1995 (JRL):
C           Original version
C
C-------------------------------------------------------------------------------
      implicit none
C
C     Global parameters
C
      include 'AF2_PAR.INC'
      include 'AF2_FSTA.INC'
C
C     Parameters
C
      integer nobj
      integer npivots
      integer nsky
      real object(2,*)
      integer object_ref(*)
      integer otype(*)
      character plog(*)*80
      real weight(*)
      integer n_obj_left
      real skypos(2,*)
      integer sky_ref(*)
      integer n_sky_left
      character slog(*)*80
      integer fib(2,npivots)
      integer piv(2,npivots)
      integer park(2,npivots)
      integer fibstatus(npivots)
      integer allocated(npivots)
      character label*(*)
      integer piv_r
      real maxnonrad
      real fibrelen2
      integer fibdims(3)
      real field_size
      real pgrad
      real pscale(2)
      integer status
C
C     Local variables
C
      integer undo_ptr,nundo,nunun,undo_buf(7,maxundo),cur_obj,len1
      integer length,chosen,pgstat,ival,oref,fromno,fromx,fromy
      integer tono,tox,toy,i,cur_fib,nop,nsp,nfp,ncp
      real plotlim(4),x,y,plothsize,x2,y2,but1(2),piv1(2)
      real button(8),tube(8),sumw
      character cur*1,string*128,softdev*32,value*3,cur2*1
      logical ok,undoundo,quit
C
C     Functions
C
      integer findobj,pgbeg,findfib
      real sumweights
C 
C     Inherited status
C
      if (status .ne. af2_ok) go to 500
C
C     Initialise the undo buffer. The variable 'undo_ptr' will give
C     the location in the buffer where the next undo operation is defined.
C     The variable nundo gives the total number of undo operations 
C     currently defined. (Obviously this has a maximum of maxundo).
C     The variable nunun gives a similar value for undoundo operations
C
      call init_undo(7,maxundo,undo_ptr,nundo,nunun,undo_buf)
C
C     Set initial plot limits so that the whole frame is in the field
C
      plotlim(1) = -pgrad
      plotlim(2) = pgrad
      plotlim(3) = -pgrad
      plotlim(4) = pgrad
C
C     Initialise the restore buffers.  This is to allow you to reset the
C     configuration back to the way it was before you started editing.
C     This call allocates some workspace and copies the input configuration
C     to it.
C
      call restore(n_obj_left,object_ref,n_sky_left,sky_ref,npivots,
     :   fib,allocated,1,status)
      if (status .ne. af2_ok) go to 500
C
C     Put the cursor at the centre of the frame initially.
C
      x = 0.0
      y = 0.0
C
C     Now loop until the user types 'E' (exit)
C
      quit = .false.
      do while (.not. quit)
C
C        Get the cursor keystroke
C
         call cursor(x,y,cur,status)
         if (status .ne. af2_ok) go to 500
C
C        ***I:  Give the user some information about an object
C
         if (cur .eq. 'I') then
C
C           First find the object which was indicated by the cursor
C
            cur_obj = findobj(nobj,object,object_ref,nsky,skypos,
     :         sky_ref,.false.,x,y)
C
C           Find if this object has been covered by a fibre
C
            i = 1
            do while ((i .le. npivots) .and. 
     :         (cur_obj .ne. allocated(i)))
               i = i + 1
            end do
            cur_fib = i
C
C           Create a message about this object...
C
            len1 = 0
            if (cur_obj .lt. alloc_sky) then
               call objinfo(cur_obj,plog,otype(cur_obj),string,len1)
            else
               call objinfo(cur_obj,slog,3,string,len1)
            end if
            if (cur_fib .le. npivots) then
               call chr_putc(' Covered by fibre ',string,len1)
               call chr_puti(cur_fib,string,len1)
            end if
C
C           Now send out a message about the object/sky
C
            call graphmsg(1,string(:len1),1,pscale)
C
C        ***R:  Refresh the current plot.  This redraws everything except the
C               the fibres since they never get drawn over.
C
         else if (cur .eq. 'R') then
            call plotit(piv_r,field_size,nobj,object,npivots,park,
     :         pgrad,otype,fibstatus,nsky,skypos,.false.,.false.,pscale,
     :         piv,fib,fibdims,.false.,label,status)
            if (status .ne. af2_ok) go to 500
C
C        ***Y:  Restore the configuration to the way it was before you
C               started the current editing session.  Copy over the original
C               configuration, replot the frame, reinitialise the undo 
C               buffer and reset the plot limits.
C
         else if (cur .eq. 'Y') then
            call restore(n_obj_left,object_ref,n_sky_left,sky_ref,
     :         npivots,fib,allocated,2,status)
            call pgpage
            call plotit(piv_r,field_size,nobj,object,npivots,park,
     :         pgrad,otype,fibstatus,nsky,skypos,.true.,.false.,pscale,
     :         piv,fib,fibdims,.true.,label,status)
            if (status .ne. af2_ok) go to 500
            call init_undo(7,maxundo,undo_ptr,nundo,nunun,undo_buf)
            plotlim(1) = -pgrad
            plotlim(2) = pgrad
            plotlim(3) = -pgrad
            plotlim(4) = pgrad
C
C        ***S:  Give the status of the configuration (how many of each type
C               of object have been configured).
C
         else if (cur .eq. 'S') then
            call count_pl(npivots,allocated,fibstatus,nop,nsp,nfp,ncp)
C
C           Write a message for each type of object and put it to the 
C           message screen.
C
            if (nop .ge. 0) then
               len1 = 0
               call chr_puti(nop,string,len1)
               call chr_putc(' objects placed',string,len1)
               call graphmsg(1,string(:len1),1,pscale)
            end if
            if (nfp .ge. 0) then
               len1 = 0
               call chr_puti(nfp,string,len1)
               call chr_putc(' fiducials placed',string,len1)
               call graphmsg(-1,string(:len1),1,pscale)
            end if
            if (nsp .ge. 0) then
               len1 = 0
               call chr_puti(nsp,string,len1)
               call chr_putc(' skies placed',string,len1)
               call graphmsg(-1,string(:len1),1,pscale)
            end if
            if (ncp .ge. 0) then
               len1 = 0
               call chr_puti(ncp,string,len1)
               call chr_putc(' cursor points placed',string,len1)
               call graphmsg(-1,string(:len1),1,pscale)
            end if
C
C           Compute the total sum of the weights of the allocated objects and
C           put that result to the message screen too.
C
            len1 = 0
            call chr_putc('Total sum of weights: ',string,len1)
            sumw = sumweights(npivots,allocated,weight)
            call chr_putr(sumw,string,len1)
            call graphmsg(-1,string(:len1),1,pscale)
C
C        ***H:  Make a hard copy of the current plot.
C
         else if (cur .eq. 'H') then
C
C           Make a note of the current soft device and close it down.
C  
            call pgqinf('DEV/TYPE',softdev,len1)
            call pgend
C
C           Request a hard copy device.  If the requested device is not
C           a hard device, then simply send an error message and skip
C           the plot.
C
            if (pgbeg(0,'?',1,1) .ne. 1) then
               status = af2_err
               go to 500
            end if
            call pgqinf('hardcopy',value,length)
            if (length .eq. 2) then
               write(*,fmt='(a)')
     :            'Requested PGPLOT device is not a hard device!'
C
C           If the device is ok, the do the plot
C
            else
               call plotit(piv_r,field_size,nobj,object,npivots,park,
     :            pgrad,otype,fibstatus,nsky,skypos,.true.,.true.,
     :            pscale,piv,fib,fibdims,.true.,label,status)
            end if
            call pgend
C
C           Now reopen the old soft device and replot
C
            if (pgbeg(0,softdev(:len1),1,1) .ne. 1) then
               status = af2_err
               go to 500
            end if
            call pgask(.false.)
            call pgvport(0.38*pscale(1),0.99,0.1,0.92)
            call pgwnad(plotlim(1),plotlim(2),plotlim(3),plotlim(4))
            call plotit(piv_r,field_size,nobj,object,npivots,park,
     :         pgrad,otype,fibstatus,nsky,skypos,.false.,.false.,pscale,
     :         piv,fib,fibdims,.true.,label,status)
C
C        ***Z: Zoom in on the region around the cursor position
C        ***O: Zoom out from region around the cursor position
C        ***X: Maximise the view (Zoom out the maximum size)
C
         else if ((cur .eq. 'Z') .or. (cur .eq. 'O') .or.
     :      (cur .eq. 'X')) then
C
C           Make sure the cursor is within the plot limits before zooming
C
            ok = .true.
            if (cur .ne. 'X') then
               if ((x .lt. plotlim(1)) .or. (x .gt. plotlim(2)) .or.
     :            (y .lt. plotlim(3)) .or. (y .gt. plotlim(4))) then
                  len1 = 0
                  call chr_putc('Cursor is outside plot window',string,
     :               len1)
                  call graphmsg(1,string(:len1),2,pscale)
                  ok = .false.
               end if
            end if
C
C           Check that the plotsize isn't smaller than one micron in WCS.
C           if it is and a zoom has been requested, refuse to do one.
C
            if ((cur .eq. 'Z') .and. (ok)) then
               if ((plotlim(2) - plotlim(1)) .le. 1.0) then
                  ok = .false.
                  len1 = 0
                  call chr_putc('Cannot zoom plot any further',string,
     :               len1)
                  call graphmsg(1,string(:len1),2,pscale)
               end if
            end if
C
C           Continue so long as you passed the previous 2 tests...
C
            if (ok) then
C
C              Get the horizontal size of the current plot
C
               plothsize = plotlim(2) - plotlim(1)
C
C              If zooming in then define a region around the cursor which 
C              is 1/4 the current region size.
C
               if (cur .eq. 'Z') plothsize = 0.25*plothsize
C
C              Redefine the plot limits.  If you are zooming out and the current
C              plot is already 80% of the maximum, then just go straight to
C              the maximum.
C
               if (((cur .eq. 'O') .and. (plothsize .gt. 0.8*pgrad)) 
     :            .or. (cur .eq. 'X')) then
                  plotlim(1) = -pgrad
                  plotlim(2) = pgrad
                  plotlim(3) = -pgrad
                  plotlim(4) = pgrad
               else
                  plotlim(1) = max(x-plothsize,-pgrad)
                  plotlim(2) = min(x+plothsize,pgrad)
                  plotlim(3) = max(y-plothsize,-pgrad)
                  plotlim(4) = min(y+plothsize,pgrad)
               end if
C
C              Go to the next page, redefine the viewport and window and
C              do the plot
C
               call pgpage
               call pgvport(0.38*pscale(1),0.99,0.1,0.92)
               call pgwnad(plotlim(1),plotlim(2),plotlim(3),plotlim(4))
               call plotit(piv_r,field_size,nobj,object,npivots,park,
     :            pgrad,otype,fibstatus,nsky,skypos,.false.,.false.,
     :            pscale,piv,fib,fibdims,.true.,label,status)
            end if
C
C        ***F:  Get information on a fibre whose button is nearest the cursor
C        ***#:  Get information on a fibre by its number
C
         else if ((cur .eq. 'F') .or. (cur .eq. '#')) then
C
C           If the key stroke was 'F', then find which is the closest fibre
C
            ok = .true.
            if (cur .eq. 'F') then
               chosen = findfib(npivots,fib,x,y)
C
C           Otherwise get the fibre number.  This is achieved by the user
C           typing the number in cursor mode and then carriage return.
C
            else
               call graphmsg(-1,'Type number and <cr> when finished',3,
     :            pscale)
               chosen = 0
               pgstat = 0
               do while (pgstat .eq. 0) 
                  call cursor(x2,y2,cur2,status)
                  if (status .ne. af2_ok) go to 500
                  read(cur2,fmt='(i1)',iostat=pgstat) ival
                  if (pgstat .eq. 0) then
                     chosen = chosen*10
                     chosen = chosen + ival
                  end if
               end do
C
C              Check that this is a valid fibre number
C
               if ((chosen .le. 0) .or. (chosen .gt. npivots)) then
                  ok = .false.
                  call graphmsg(-1,'Illegal fibre number!',2,pscale)
               end if
            end if
C
C           If the fibre number was obtained successfully, then
C           write the message appropriate for the type of fibre.
C
            if (ok) then
               len1 = 0
               call chr_putc('Selected fibre ',string,len1)
               call chr_puti(chosen,string,len1)
               if (fibstatus(chosen) .eq. fsta_ook) then
                  call chr_putc(' (object fibre) ',string,len1)
               else if (fibstatus(chosen) .eq. fsta_fok) then
                  call chr_putc(' (fiducial fibre) ',string,len1)
               end if
               oref = allocated(chosen)
C
C              Message for parked fibre
C
               if (oref .eq. 0) then
                  call chr_putc('parked',string,len1)
C
C              Message for fibre placed on cursor location
C
               else if (oref .eq. alloc_cur) then
                  call chr_putc('placed on cursor location',string,len1)
C
C              Message for allocated fibre
C
               else if (oref .gt. 0) then
                  if (oref .gt. alloc_sky) then
                     call objinfo(oref,slog,3,string,len1)
                  else
                     call objinfo(oref,plog,otype(oref),string,len1)
                  end if
C
C              Message for unavailable fibre
C
               else 
                  call chr_putc
     :               (' Fibre not available (broken, disabled etc.)',
     :               string,len1)
               end if
C
C              Now send out the message
C
               call graphmsg(1,string(:len1),1,pscale)
            end if
C
C        ***P:  Park the fibre nearest the cursor
C        ***C:  Move the fibre nearest the cursor to another point defined
C               by a second keystroke.
C        ***M:  Move the fibre nearest the cursor to an object indicated
C               by a second keystroke
C
         else if ((cur .eq. 'P') .or. (cur .eq. 'C') .or.
     :      (cur .eq. 'M')) then
            call movefibre(x,y,npivots,fib,piv,park,nobj,n_obj_left,
     :         object,object_ref,otype,plog,nsky,n_sky_left,skypos,
     :         sky_ref,slog,allocated,fibstatus,fibdims,fibrelen2,
     :         maxnonrad,undo_ptr,nundo,nunun,undo_buf,cur,pscale,
     :         status)
            if (status .ne. af2_ok) go to 500
C
C        ***U:  Undo the most recent change
C        ***A:  Undo the most recent undo
C
         else if ((cur .eq. 'U') .or. (cur .eq. 'A')) then
            undoundo = cur .eq. 'A'
            ok = .true.
C
C           If this is just a plain undo then check that you haven't
C           already run over the buffer. If you haven't then set the
C           'from' and 'to' information.  Decrement the pointer and
C           and the number of available undo commands. Increment
C           the number of undoundos.
C
            if (.not. undoundo) then
               if (nundo .eq. 0) then
                  call graphmsg(1,'No more undo information',2,pscale)
                  ok = .false.
               else
                  chosen = undo_buf(1,undo_ptr)
                  fromno = undo_buf(2,undo_ptr)
                  fromx = undo_buf(3,undo_ptr)
                  fromy = undo_buf(4,undo_ptr)
                  tono = undo_buf(5,undo_ptr)
                  tox = undo_buf(6,undo_ptr)
                  toy = undo_buf(7,undo_ptr)
                  undo_ptr = undo_ptr - 1
                  if (undo_ptr .le. 0) undo_ptr = maxundo
                  nundo = nundo - 1
                  nunun = min(nunun+1,maxundo)
               end if
C
C           If this is an undoundo then check that there is something
C           in the undo buffer for you to do. If there is, then set the
C           'from' and 'to' information and increment the pointer and
C           the number of undo commands.  Decrement the number of undoundos
C
            else
               if (nunun .eq. 0) then
                  call graphmsg(1,'No more undoundo information',2,
     :               pscale)
                  ok = .false.
               else
                  undo_ptr = undo_ptr + 1
                  if (undo_ptr .gt. maxundo) undo_ptr = 1
                  nundo = min(nundo+1,maxundo)
                  nunun = nunun - 1
                  chosen = undo_buf(1,undo_ptr)
                  tono = undo_buf(2,undo_ptr)
                  tox = undo_buf(3,undo_ptr)
                  toy = undo_buf(4,undo_ptr)
                  fromno = undo_buf(5,undo_ptr)
                  fromx = undo_buf(6,undo_ptr)
                  fromy = undo_buf(7,undo_ptr)
               end if
            end if
C
C           If we have ok'd this change then get the dimensions of the
C           button and tube as they currently stand.
C
            if (ok) then
               but1(1) = real(tox)
               but1(2) = real(toy)
               piv1(1) = real(piv(1,chosen))
               piv1(2) = real(piv(2,chosen))
               call calc_rects(piv1,but1,fibdims,button,tube)
C
C              Erase the fibre
C
               call erase(button,tube,fibstatus(chosen))
               call objover(nobj,object,otype,nsky,skypos,tube,button,
     :            fib(1,chosen),piv(1,chosen))
C
C              Now store away the information for the new object
C
               fib(1,chosen) = fromx
               fib(2,chosen) = fromy
               allocated(chosen) = fromno
C
C              Calculate new tube and button dimensions as they will be
C              when the fibre moves.
C
               but1(1) = real(fromx)
               but1(2) = real(fromy)
               call calc_rects(piv1,but1,fibdims,button,tube)
C
C              Now redraw the fibre into its new position
C
               call normfibre(button,tube,fibstatus(chosen))
               call objover(nobj,object,otype,nsky,skypos,tube,button,
     :            fib(1,chosen),piv(1,chosen))
               call pgsci(1)
C
C              Remove the new object from the reference list and add
C              the old object back in...
C
               call remove_from_list(object_ref,n_obj_left,sky_ref,
     :            n_sky_left,fromno)
               call add_to_list(object_ref,n_obj_left,sky_ref,
     :            n_sky_left,tono)
            end if
C
C        ***E:  Exit from the editing loop
C
         else if (cur .eq. 'E') then
            quit = .true.
C
C        ***Unknown key stroke -- send an error message...
C
         else
            call graphmsg(1,'Unknown option',2,pscale)
         end if
C
C     ***End of editing do-while loop
C
      end do
C
C     Free the restore buffer workspace
C
      call restore(n_obj_left,object_ref,n_sky_left,sky_ref,npivots,
     :   fib,allocated,3,status)
C
C     Exit
C
  500 continue
      end
