C+------------------------------------------------------------------------------
C
C     editsubs -- A suite of subroutines and functions used by the 
C                 configuration editor.  The following routines are 
C                 included here:
C
C                    findobj -- locate object closest to given x,y position
C                    findfib -- locate fibre closest to given x,y position
C                    objinfo -- create string of info about an object
C                    highlight -- highlight a fibre on the plot
C                    erase -- erase a fibre on the plot
C                    normfibre -- draw a fibre in normal colours on the plot
C                    init_undo -- initialise the undo buffer
C
C-------------------------------------------------------------------------------


      integer function findobj(nobj,object,object_ref,nsky,
     :   skypos,sky_ref,useref,x,y)
C+------------------------------------------------------------------------------
C
C     Name:   
C        findobj
C
C     Invocation:
C        object = findobj(nobj,object,object_ref,nsky,skypos,
C           sky_ref,useref,x,y)
C
C     Description:
C        The object and sky lists are searched for an object or
C        sky patch nearest an input x,y location
C
C     Arguments: (">" Input, "<" Output, "W" Workspace, "!"  Changed)
C        nobj:  Integer (>)
C           The number of objects.  If useref is true, then this should
C           be the number of objects left in object_ref.  Otherwise it
C           should be the total number of objects in the object array.
C        object:  Real array (>)
C           The x,y coordinates of the objects (program+fiducial)
C        object_ref:  Integer array (>)
C           The list of reference numbers of objects which have not 
C           been allocated to a fibre.
C        nsky:  Integer (>)
C           The number of sky patches.  If useref is true, then this
C           should be the number of sky patches left in sky_ref.  Otherwise
C           it should be the total number of sky patches in the sky array.
C        skypos:  Real array (>)
C           The x,y coordinates of the sky patches
C        sky_ref:  Integer array (>)
C           The list of reference numbers of the sky patches which have
C           not been allocated to a fibre
C        useref:  Logical (>)
C           TRUE if we are using the object reference number rather than
C           just the index of the object array
C        x:  real (>)
C           The input X coordinate 
C        y:  real (>)
C           The input Y coordinate
C
C     Returns:
C        object: Integer
C           The number of the object or sky patch which is closest to the
C           given x,y coordinates.
C
C     Language:
C        FORTRAN 77
C
C     Non-standard language extensions:
C        include, implicit none, variable names > 6 characters
C
C     Logical i/o units used:
C        none
C
C     External routines called:
C        none
C 
C     Algorithm:
C        -- If there are any objects, then the distance to the first object
C           in the list from x,y is taken as the initial guess of the minimum
C        -- The distance to the rest of the objects is calculated and the
C           minimum is found.
C        -- If there are sky patches, then if there were objects loop
C           through all the sky patches and see if any of them are closer
C           to x,y than the closest object.  If there weren't any objects,
C           make sure you use the first sky patch as the initial guess of
C           the minimum distance before looking at the rest of the patches.
C
C     Authors:
C        (JRL): James R. Lewis -- RGO, Cambridge
C
C     History:
C        01 Dec 1995 (JRL):
C           Original version
C
C-------------------------------------------------------------------------------
      implicit none
C
C     Global parameters
C
      include 'AF2_PAR.INC'
C
C     Parameters
C
      integer nobj
      real object(2,*)
      integer object_ref(*)
      integer nsky
      real skypos(2,*)
      integer sky_ref(*)
      logical useref
      real x
      real y
C
C     Local variables
C
      integer i,curobj,oref
      real value,value2
C
C     Initialise the output object number
C
      curobj = 0
C
C     Start by looking through the object list (if there are any).  Find
C     the closest object to the input x,y position.
C
      if (nobj .gt. 0) then
         oref = 1
         if (useref) oref = object_ref(1)
         value2 = (x - object(1,oref))**2 + (y - object(2,oref))**2
         curobj = 1
         do 10 i = 2,nobj
            oref = i
            if (useref) oref = object_ref(i)
            value = (x - object(1,oref))**2 + (y - object(2,oref))**2
            if (value .lt. value2) then
               curobj = oref
               value2 = value
            end if
  10     continue
      end if
C
C     Now look through the sky patch list and see if there are any of
C     these which are closer than the closest object.  If there weren't any
C     objects (unlikely though that is...) use the first sky patch as the
C     initial guess.
C
      if (nsky .gt. 0) then
         if (nobj .le. 0) then
            oref = 1
            if (useref) oref = sky_ref(1)
            value2 = (x - skypos(1,oref))**2 + (y - skypos(2,oref))**2
            curobj = alloc_sky + oref
         end if
         do 20 i = 1,nsky
            oref = i
            if (useref) oref = sky_ref(i)
            value = (x - skypos(1,oref))**2 + (y - skypos(2,oref))**2
            if (value .lt. value2) then
               curobj = oref + alloc_sky
               value2 = value
            end if
  20     continue
      end if
C
C     Now return the chosen object/sky
C
      findobj = curobj
      end


      integer function findfib(npivots,fib,x,y)
C+------------------------------------------------------------------------------
C
C     Name:   
C        findfib
C
C     Invocation:
C        fibnum = findfib(npivots,fib,x,y)
C
C     Description:
C        Locate the fibre whose button is closest to a give x,y position
C
C     Arguments: (">" Input, "<" Output, "W" Workspace, "!"  Changed)
C        npivots:  Integer (>)
C           The total number of pivots
C        fib:  Integer array (>)
C           The current x,y position of each fibre button
C        x:  Real (>)
C           The input X position
C        y:  Real (>)
C           The input Y position
C
C     Returns:
C        fibnum:  Integer
C           The number of the fibre whose button head is closest to the
C           input x,y coordinates.
C
C     Language:
C        FORTRAN 77
C
C     Non-standard language extensions:
C        include, implicit none, variable names > 6 characters
C
C     Logical i/o units used:
C        none
C
C     External routines called:
C        none
C 
C     Algorithm:
C        -- The distance to the first fibre button from x,y is taken
C           as the initial guess of the minimum
C        -- The rest of the fibres are tested to see if any of them are
C           closer, and if so, which is the closest.
C
C     Authors:
C        (JRL): James R. Lewis -- RGO, Cambridge
C
C     History:
C        01 Dec 1995 (JRL):
C           Original version
C
C-------------------------------------------------------------------------------
      implicit none
C
C     Parameters
C
      integer npivots
      integer fib(2,npivots)
      real x
      real y
C
C     Local variables
C
      real value,value2
      integer i,curfib
C
C     Search through the fibre positions and find the fibre which is closest
C     to the input x,y position
C
      value2 = (real(fib(1,1)) - x)**2 + (real(fib(2,1)) - y)**2
      curfib = 1
      do 10 i = 2,npivots
         value = (real(fib(1,i)) - x)**2 + (real(fib(2,i)) - y)**2
         if (value .lt. value2) then
            curfib = i
            value2 = value
         end if
  10  continue
C
C     Return the chosen fibre
C
      findfib = curfib
      end 

      subroutine objinfo(oref,plog,type,string,len1)
C+------------------------------------------------------------------------------
C
C     Name:   
C        objinfo
C
C     Invocation:
C        call objinfo(oref,plog,type,string,len1)
C
C     Description:
C        An information string is formed for an object with a certain
C        reference number.
C
C     Arguments: (">" Input, "<" Output, "W" Workspace, "!"  Changed)
C        oref:  Integer (>)
C           Reference number for the object in question
C        plog:  Character array (>)
C           The log of information on each of the objects
C        type:  Integer (>)
C           The object type (1=program, 2=fiducial, 3=sky)
C        string:  Character (!)
C           The output string with the object information
C        len1:  Integer (!)
C           The location in the string of the last non-blank character
C
C     Language:
C        FORTRAN 77
C
C     Non-standard language extensions:
C        include, implicit none, variable names > 6 characters
C
C     Logical i/o units used:
C        none
C
C     External routines called:
C        Library chr:
C           chr_appnd
C           chr_dcwrd
C           chr_putc
C           chr_puti
C 
C     Algorithm:
C        -- If the object reference is greater than the sky patch reference
C           number offset, then this is a sky patch.  Otherwise its an 
C           object (either program or fiducial).
C        -- The type of object and the object reference number are written
C           to the string.  
C        -- The information log is searched.  The object name is appended
C           to the string (word 1). If this isn't a sky and if words 9 and 
C           10 exist (the object weight and magnitude) the these are added
C           to the output string as well.
C
C     Authors:
C        (JRL): James R. Lewis -- RGO, Cambridge
C
C     History:
C        01 Dec 1995 (JRL):
C           Original version
C
C-------------------------------------------------------------------------------
      implicit none
C
C     Global parameters
C
      include 'AF2_PAR.INC'
C
C     Parameters
C
      integer oref
      character plog(*)*80
      integer type
      character string*(*)
      integer len1
C
C     Local parameters
C
      integer maxword
      parameter (maxword=10)
C
C     Local variables
C
      integer oref2,nwrds,startw(maxword),stopw(maxword),lstat
      logical sky
      character word(maxword)*40
C
C     If this is a sky object, then subtract the sky reference offset
C
      if (oref .gt. alloc_sky) then
         oref2 = oref - alloc_sky
         sky = .true.
      else
         oref2 = oref
         sky = .false.
      end if
C
C     Start the string off with the words 'object' or 'sky position' 
C     depending upon what sort we have.  Then add the object reference.
C     Finally, if this is not a sky patch, then tell the user what type
C     of object you have (either program or fiducial).
C
      if (.not. sky) then
         call chr_putc('Object ',string,len1)
      else
         call chr_putc('Sky position ',string,len1)
      end if
      call chr_puti(oref2,string,len1)
      if (type .eq. 1) then
         call chr_putc(' (program), ',string,len1)
      else if (type .eq. 2) then
         call chr_putc(' (fiducial), ',string,len1)
      else
         call chr_putc(' , ',string,len1)
      end if
C
C     Decode the log into words and add the name of the object
C
      call chr_dcwrd(plog(oref2),maxword,nwrds,startw,stopw,word,lstat)
      call chr_appnd(word(1),string,len1)
C
C     Now if the info exists and this is an object, then add the magnitude.
C     Add the weight to the string for all non-skys.
C
      if (.not. sky) then
         if ((nwrds .ge. 10) .and. (word(10)(1:1) .ne. '*')) then
            call chr_putc(' (mag = ',string,len1)
            call chr_appnd(word(10),string,len1)
            call chr_putc(', ',string,len1)
         else
            call chr_putc(' (',string,len1)
         end if
         call chr_putc('wt = ',string,len1)
         call chr_appnd(word(9),string,len1)
         call chr_putc(')',string,len1)
      end if
C
C     Exit
C
      end

      subroutine highlight(button,tube,fibstatus)
C+------------------------------------------------------------------------------
C
C     Name:   
C        highlight
C
C     Invocation:
C        call highlight(button,tube,fibstatus)
C
C     Description:
C        The plot of a fibre is written in a highlight colour appropriate
C        for the fibre type.
C
C     Arguments: (">" Input, "<" Output, "W" Workspace, "!"  Changed)
C        button:  Real array (>)
C           The x,y coordinates of the corners of the rectangle defining
C           the fibre button.
C        tube:  Real array (>)
C           The x,y coordinates of the corners of the rectangle defining
C           the fibre tube.
C        fibstatus:  Integer (>)
C           The fibre status (program, fiducial, missing, disabled, etc)
C
C     Language:
C        FORTRAN 77
C
C     Non-standard language extensions:
C        include, implicit none, variable names > 6 characters
C
C     Logical i/o units used:
C        none
C
C     External routines called:
C        Library pgplot:
C           pgpoly
C           pgqci
C           pgsci
C 
C     Algorithm:
C        -- The current colour index is queried.
C        -- The colour index is changed to show what type of fibre we
C           are dealing with.  Yellow is for program fibres, cyan is for
C           fiducal fibres.  Missing, disabled and dead fibres cannot be
C           moved, so if we have one here, then we exit immediately.
C        -- The fibre is drawn in the new colour and the colour index is
C           restored to its original value.
C
C     Authors:
C        (JRL): James R. Lewis -- RGO, Cambridge
C
C     History:
C        01 Dec 1995 (JRL):
C           Original version
C
C-------------------------------------------------------------------------------
      implicit none
C
C     Global parameters
C
      include 'AF2_FSTA.INC'
C
C     Parameters
C
      real button(8)
      real tube(8)
      integer fibstatus
C
C     Local variables
C
      integer ci
C
C     Query the current colour index 
C
      call pgqci(ci)
C
C     Set the colour index depending upon the fibre status.  If the fibre
C     is dead or disabled then you shouldn't be able to move it 
C     anyway, so get out of here if it is.  Set the fill style to solid.
C
      if (fibstatus .eq. fsta_ook) then
         call pgsci(7)
      else if (fibstatus .eq. fsta_fok) then
         call pgsci(5)
      else
         go to 500
      end if
C
C     Now highlight the fibre
C
      call pgpoly(4,button(1),button(5))
      call pgpoly(4,tube(1),tube(5))
C
C     Return the colour index to its original state
C
      call pgsci(ci)
C
C     Exit
C
  500 continue
      end

      subroutine erase(button,tube,fibstatus)
C+------------------------------------------------------------------------------
C
C     Name:   
C        erase
C
C     Invocation:
C        call erase(button,tube,fibstatus)
C
C     Description:
C        The plot of a fibre is erased by drawing over it in the background 
C        colour.
C
C     Arguments: (">" Input, "<" Output, "W" Workspace, "!"  Changed)
C        button:  Real array (>)
C           The x,y coordinates of the corners of the rectangle defining
C           the fibre button.
C        tube:  Real array (>)
C           The x,y coordinates of the corners of the rectangle defining
C           the fibre tube.
C        fibstatus:  Integer (>)
C           The fibre status (program, fiducial, missing, disabled, etc)
C
C     Language:
C        FORTRAN 77
C
C     Non-standard language extensions:
C        include, implicit none, variable names > 6 characters
C
C     Logical i/o units used:
C        none
C
C     External routines called:
C        Library pgplot:
C           pgpoly
C           pgqci
C           pgsci
C 
C     Algorithm:
C        -- Missing, disabled and dead fibres cannot be moved, so if 
C           we have one here, then we exit immediately.
C        -- The current colour index is queried.
C        -- The colour index is changed to the background colour (color
C           index = 0) and the fibre is drawn (erased).
C        -- The colour index is restored to its original value.
C
C     Authors:
C        (JRL): James R. Lewis -- RGO, Cambridge
C
C     History:
C        01 Dec 1995 (JRL):
C           Original version
C
C-------------------------------------------------------------------------------
      implicit none
C
C     Global parameters
C
      include 'AF2_FSTA.INC'
C
C     Parameters
C
      real button(8)
      real tube(8)
      integer fibstatus
C
C     Local variables
C
      integer ci
C
C     If this is not a moveable fibre, then get out of here
C
      if ((fibstatus .ne. fsta_ook) .and. 
     :   (fibstatus .ne. fsta_fok)) go to 500
C
C     Query the colour index 
C
      call pgqci(ci)
C
C     'Erase' the fibre (i.e. draw it in the background colour)
C
      call pgsci(0)
      call pgpoly(4,button(1),button(5))
      call pgpoly(4,tube(1),tube(5))
C
C     Return the old colour index 
C
      call pgsci(ci)
C
C     Exit
C
  500 continue
      end

      subroutine normfibre(button,tube,fibstatus)
C+------------------------------------------------------------------------------
C
C     Name:   
C        normfibre
C
C     Invocation:
C        call normfibre(button,tube,fibstatus)
C
C     Description:
C        The plot of a fibre is drawn in the colour defined for the
C        fibre's particular type
C
C     Arguments: (">" Input, "<" Output, "W" Workspace, "!"  Changed)
C        button:  Real array (>)
C           The x,y coordinates of the corners of the rectangle defining
C           the fibre button.
C        tube:  Real array (>)
C           The x,y coordinates of the corners of the rectangle defining
C           the fibre tube.
C        fibstatus:  Integer (>)
C           The fibre status (program, fiducial, missing, disabled, etc)
C
C     Language:
C        FORTRAN 77
C
C     Non-standard language extensions:
C        include, implicit none, variable names > 6 characters
C
C     Logical i/o units used:
C        none
C
C     External routines called:
C        Library pgplot:
C           pgpoly
C           pgqci
C           pgsci
C 
C     Algorithm:
C        -- Missing fibres cannot be drawn, so if we have one here, 
C           then we exit immediately.
C        -- The current colour index is queried.
C        -- The colour index is changed to the reflect the fibre type
C           (program = green, fiducial = blue, disabled prog = red,
C           disabled fid = magenta, dead prog = red+magenta, 
C           dead fid = blue+magenta) and the fibre is drawn
C        -- The colour index is restored to its original value.
C
C     Authors:
C        (JRL): James R. Lewis -- RGO, Cambridge
C
C     History:
C        01 Dec 1995 (JRL):
C           Original version
C
C-------------------------------------------------------------------------------
      implicit none
C
C     Global parameters
C
      include 'AF2_FSTA.INC'
C
C     Parameters
C
      real button(8)
      real tube(8)
      integer fibstatus
C
C     Local variables
C
      integer ci
C
C     If the fibre is missing, then get out of here
C
      if (fibstatus .eq. fsta_miss) go to 500
C
C     Get the old colour index 
C
      call pgqci(ci)
C
C     Set the colour index depending upon the fibre status 
C
      if (fibstatus .eq. fsta_ook) then
         call pgsci(3)
      else if (fibstatus .eq. fsta_fok) then
         call pgsci(4)
      else if (fibstatus .eq. fsta_odis) then
         call pgsci(2)
      else if (fibstatus .eq. fsta_fdis) then
         call pgsci(6)
      else if (fibstatus .eq. fsta_odead) then
         call pgsci(13)
      else if (fibstatus .eq. fsta_fdead) then
         call pgsci(12)
      else
         go to 500
      end if
C
C     Draw the fibre
C
      call pgpoly(4,button(1),button(5))
      call pgpoly(4,tube(1),tube(5))
C
C     Reset the colour index 
C
      call pgsci(ci)
C
C     Exit
C
  500 continue
      end

      subroutine init_undo(m,n,undo_ptr,nundo,nunun,undo_buf)
C+------------------------------------------------------------------------------
C
C     Name:   
C        init_undo
C
C     Invocation:
C        call init_undo(m,n,undo_ptr,nundo,nunun,undo_buf)
C
C     Description:
C        The editor's undo buffer is initialised
C
C     Arguments: (">" Input, "<" Output, "W" Workspace, "!"  Changed)
C        m:  Integer (>)
C           The first dimension of the undo buffer array
C        n:  Integer (>)
C           The second dimension of the undo buffer array
C        undo_ptr:  Integer (!)
C           The location in the buffer of the most recent action
C        nundo:  Integer (!)
C           The number of undo actions available
C        nunun:  Integer (!)
C           The number of undoundo actions available
C        undo_buf:  Integer array (!)
C           The undo buffer. (What is in each element is not important here
C           since all we are doing is zeroing everything...)
C
C     Language:
C        FORTRAN 77
C
C     Non-standard language extensions:
C        implicit none, variable names > 6 characters
C
C     Logical i/o units used:
C        none
C
C     External routines called:
C        none
C 
C     Algorithm:
C        -- The undo buffer is zeroed as are the undo pointer, and
C           the two action counters.
C
C     Authors:
C        (JRL): James R. Lewis -- RGO, Cambridge
C
C     History:
C        01 Dec 1995 (JRL):
C           Original version
C
C-------------------------------------------------------------------------------
      implicit none
C
C     Parameters
C
      integer m
      integer n
      integer undo_ptr
      integer nundo
      integer nunun
      integer undo_buf(m,n)
C
C     Local variables
C
      integer i,j
C
C     Zero the undo buffer array
C
      do 20 j = 1,n
         do 10 i = 1,m
            undo_buf(i,j) = 0
  10     continue
  20  continue
C
C     Initialise the undo counters and the undo pointer
C
      nundo = 0
      nunun = 0
      undo_ptr = 0
C
C     Exit
C
      end         
