      subroutine movefibre(x,y,npivots,fib,piv,park,nobj,n_obj_left,
     :   object,object_ref,otype,plog,nsky,n_sky_left,skypos,sky_ref,
     :   slog,allocated,fibstatus,fibdims,fibrelen2,maxnonrad,undo_ptr,
     :   nundo,nunun,undo_buf,cur,pscale,status)
C+------------------------------------------------------------------------------
C
C     Name:   
C        movefibre
C
C     Invocation:
C        call movefibre(x,y,npivots,fib,piv,park,nobj,n_obj_left,
C           object,object_ref,otype,plog,nsky,n_sky_left,skypos,sky_ref,
C           slog,allocated,fibstatus,fibdims,fibrelen2,maxnonrad,undo_ptr,
C           nundo,nunun,undo_buf,cur,pscale,status)
C
C     Description:
C        Move the fibre which is closest to the given x,y position.
C        This fibre can be either parked, moved to a particular object
C        or moved to a second cursor position.  Draw the modified position.
C
C     Arguments: (">" Input, "<" Output, "W" Workspace, "!"  Changed)
C        x:  Real (>)
C           The cursor X position
C        y:  Real (>)
C           The cursor Y position
C        npivots:  Integer (>)
C           The total number of pivots
C        fib:  Integer array (!)
C           The current x,y position of each fibre button
C        piv:  Integer array (>)
C           The x,y position of each of the pivots
C        park:  Integer array (>)
C           The park position for each of the fibres
C        nobj:  Integer (>)
C           The total number of objects
C        n_obj_left:  Integer (!)
C           The number of objects which have not yet been allocated.
C        object:  Real array (>)
C           The x,y coordinates of the objects (program+fiducial)
C        object_ref:  Integer array (!)
C           The list of reference numbers of objects which have not 
C           been allocated to a fibre.
C        otype:  Integer array (>)
C           The object type (i.e. whether it's a program or sky)
C        plog:  Character array (>)
C           The log of information on each of the objects
C        nsky:  Integer (>)
C           The total number of sky patches
C        n_sky_left:  Integer (!)
C           The number of sky patches which have not yet been allocated
C        skypos:  Real array (>)
C           The x,y coordinates of the sky patches
C        sky_ref:  Integer array (!)
C           The list of reference numbers of the sky patches which have
C           not been allocated to a fibre
C        slog:  Character array (>)
C           The log of information on each of the sky patches
C        allocated:  Integer array (>)
C           The reference number of the object which has been allocated
C           to a particular fibre.
C        fibstatus:  Integer array (>)
C           The fibre status (i.e. whether it's useable, disabled etc.)
C        fibdims:  Integer array (>)
C           The dimensions of the fibres: 1 -- button length, 2 -- button
C           width, 3 -- tube width
C        fibrelen2:  Real (>)
C           The square of length of a fibre
C        maxnonrad: Real (>)
C           The maximum non-radial angle each fibre is allowed to subtend
C        undo_ptr:  Integer (!)
C           The location in the buffer of the most recent action
C        nundo:  Integer (!)
C           The number of undo actions available
C        nunun:  Integer (!)
C           The number of undoundo actions available
C        undo_buf:  Integer array (!)
C           The undo buffer. The elements are: 1 -- fibre number
C           2 -- reference number of former object, 3 -- X coordinate
C           of former location, 4 -- Y coordinate of former location,
C           5 -- reference number of new object, 6  -- X coordinate
C           of new location, 4 -- Y coordinate of new location.
C        cur:  Character (>)
C           The original cursor keystroke value
C        pscale:  Real array (>)
C           Plot scaling factors: 1 -- the linear scale, 2 -- the text height
C        status:  Integer (!)
C           The global status variable
C
C     Language:
C        FORTRAN 77
C
C     Non-standard language extensions:
C        include, implicit none, variable names > 6 characters
C
C     Logical i/o units used:
C        none
C
C     External routines called:
C        Library subaf2_configure:
C           add_to_list
C           calc_rects
C           cursor
C           do_fibre_check
C           erase
C           findfib
C           findobj
C           graphmsg
C           highlight
C           mkmessage
C           normfibre
C           objinfo
C           objover
C           remove_from_list
C 
C        Library chr:
C           chr_putc
C           chr_puti
C           chr_putr
C 
C        Library pgplot:
C           pgsci
C 
C     Algorithm:
C        -- Fibres can either be parked (cur='P'), placed at a second cursor
C           position (cur='C') or moved to a specific object (cur='M')
C        -- Find the fibre whose button is nearest the cursor position
C           and highlight it.
C        -- Get the destination location. For 'P' just take the park position
C           of the current fibre.  For 'C' get another keystroke and make sure
C           it's inside the park circle.  For 'M' get another keystoke and 
C           find out which object is the nearest.  Write a message about the 
C           object and make sure the object and the fibre are compatible types.
C        -- Once we have a destination position, make sure that it won't collide
C           with any of the other fibres in the field.
C        -- If there is no collision, then the fibre is erased from its old
C           position.  The formerly allocated object is returned to the 
C           object reference list and the undo buffer is given the information
C           on the old object.
C        -- The fibre is drawn in its new location and an appropriate message
C           is written according to the action which has been taken.
C        -- The newly allocated object is now removed from the object reference
C           list and the new object information is added to the undo buffer.
C
C     Authors:
C        (JRL): James R. Lewis -- RGO, Cambridge
C
C     History:
C        01 Dec 1995 (JRL):
C           Original version
C
C-------------------------------------------------------------------------------
      implicit none
C
C     Global parameters
C
      include 'AF2_PAR.INC'
      include 'AF2_FCHK.INC'
      include 'AF2_FSTA.INC'
C
C     Parameters
C
      real x
      real y
      integer npivots
      integer fib(2,npivots)
      integer piv(2,npivots)
      integer park(2,npivots)
      integer nobj
      integer n_obj_left
      real object(2,*)
      integer object_ref(*)
      integer otype(*)
      character plog(*)*80
      integer nsky
      integer n_sky_left
      real skypos(2,*)
      integer sky_ref(*)
      character slog(*)*80
      integer allocated(npivots)
      integer fibstatus(npivots)
      integer fibdims(3)
      real fibrelen2
      real maxnonrad
      integer undo_ptr
      integer nundo
      integer nunun
      integer undo_buf(7,maxundo)
      character cur*(*)
      real pscale(2)
      integer status
C
C     Local variables
C
      integer chosen,len1,cur_obj,oref,offender,fstatus,lenm
      character string*80,cur2*1,message*80
      logical dopark,docurs,dospec,ok
      real piv1(2),but1(2),x2,y2,rad1,rad2,button(8),tube(8)
C
C     Functions
C
      integer findfib,findobj
C
C     Inherited status
C
      if (status .ne. af2_ok) go to 500
C
C     Decide exactly what it is the user wanted to do: park, move to
C     a cursor position or move to a specific object
C
      dopark = .false.
      docurs = .false.
      dospec = .false.
      if (cur(1:1) .eq. 'P') then
         dopark = .true.
      else if (cur(1:1) .eq. 'C') then
         docurs = .true.
      else if (cur(1:1) .eq. 'M') then
         dospec = .true.
      end if
C
C     Find out which fibre the user had in mind
C
      chosen = findfib(npivots,fib,x,y)
C
C     Tell the user which one he/she/it chose.
C
      len1 = 0
      call chr_putc('Selected fibre ',string,len1)
      call chr_puti(chosen,string,len1)
      call graphmsg(1,string(:len1),1,pscale)
C
C     If this is a fibre which can't be moved (disabled or missing) then
C     get out of here immediately
C
      if ((fibstatus(chosen) .ne. fsta_ook) .and. 
     :   (fibstatus(chosen) .ne. fsta_fok)) then
         call graphmsg(-1,'Cannot move disabled/missing fibres',2,
     :      pscale)
         go to 500
      end if
C
C     Calculate the dimensions of the chosen fibre
C
      piv1(1) = real(piv(1,chosen))
      piv1(2) = real(piv(2,chosen))
      but1(1) = real(fib(1,chosen))
      but1(2) = real(fib(2,chosen))
      call calc_rects(piv1,but1,fibdims,button,tube)
C
C     Now highlight the chosen fibre
C
      call highlight(button,tube,fibstatus(chosen))
      call objover(nobj,object,otype,nsky,skypos,tube,button,
     :   fib(1,chosen),piv(1,chosen))
C
C     If you have requested to park the fibre, then put the correct park
C     position into the button location.  If the fibre is already parked
C     then send out a brusque message and get out of here.
C
      if (dopark) then
         if (allocated(chosen) .eq. 0) then
            call graphmsg(-1,'Fibre is already parked!',2,pscale)
            call normfibre(button,tube,fibstatus(chosen))
            call objover(nobj,object,otype,nsky,skypos,tube,button,
     :         fib(1,chosen),piv(1,chosen))
            go to 500
         end if
         but1(1) = real(park(1,chosen))
         but1(2) = real(park(2,chosen))
C
C     If you have requested to move the fibre to a chosen cursor location, 
C     then get the desired location with the cursor.
C
      else if (docurs) then
         call graphmsg(-1,'Mark a place to set the fibre',1,pscale)
         x2 = but1(1)
         y2 = but1(2)
         call cursor(x2,y2,cur2,status)
         if (status .ne. af2_ok) go to 500
C
C        If the chosen fibre is a fiducial fibre, send a warning message.
C        This is because such fibres are not of use for observing sky, but
C        the user may be moving it so that it is out of the way of a program
C        fibre.
C
         if (fibstatus(chosen) .eq. fsta_fok) then
            call graphmsg(-1,
     :         'Fibre not suitable for sky, but will still move it',1,
     :         pscale)
         end if
C
C        Check to make sure you are not trying to move the fibre outside the
C        park circle.  If you are, then send an error messsage, put the
C        fibre back to its proper colour and get out of here.
C
         rad1 = x2**2 + y2**2
         rad2 = real(park(1,chosen))**2 + real(park(2,chosen))**2
         if (rad1 .gt. rad2) then
            call graphmsg(-1,'Cannot move fibre outside park circle',
     :         2,pscale)
            call normfibre(button,tube,fibstatus(chosen))
            call objover(nobj,object,otype,nsky,skypos,tube,button,
     :         fib(1,chosen),piv(1,chosen))
            go to 500
         end if
C
C        Otherwise keep the new x,y position as the destination of the fibre
C
         but1(1) = x2
         but1(2) = y2
C
C     Finally, if you are moving to a specific object, mark the object
C     and search the object and sky list for it.
C
      else if (dospec) then
         call graphmsg(-1,'Mark the object',1,pscale)
         x2 = x
         y2 = y
         call cursor(x2,y2,cur2,status)
         if (status .ne. af2_ok) go to 500
         cur_obj = findobj(n_obj_left,object,object_ref,n_sky_left,
     :      skypos,sky_ref,.true.,x2,y2)
C
C        Now form a message about the object 
C
         len1 = 0
         if (cur_obj .lt. alloc_sky) then
            call objinfo(cur_obj,plog,otype(cur_obj),string,len1)
         else
            call objinfo(cur_obj,slog,3,string,len1)
         end if
C
C        Add the positional information to the string and write it out
C
         if (cur_obj .lt. alloc_sky) then
            but1(1) = object(1,cur_obj)
            but1(2) = object(2,cur_obj)
         else
            but1(1) = skypos(1,cur_obj-alloc_sky)
            but1(2) = skypos(2,cur_obj-alloc_sky)
         end if
         call chr_putc(' at ',string,len1)
         call chr_putr(but1(1),string,len1)
         call chr_putc(',',string,len1)
         call chr_putr(but1(2),string,len1)
         call graphmsg(-1,string(:len1),1,pscale)
C
C        If this is the wrong object/fibre combination, then signal an error
C
         if (cur_obj .gt. alloc_sky) then
            oref = cur_obj - alloc_sky
            if (fibstatus(chosen) .ne. fsta_ook) then
               ok = .false.
            else 
               ok = .true.
            end if
         else 
            oref = cur_obj
            if (fibstatus(chosen) .ne. otype(oref)) then
               ok = .false.
            else
               ok = .true.
            end if
         end if
         if (.not. ok) then
            call graphmsg(-1,'Wrong sort of object/fibre pair',2,
     :         pscale)
            call normfibre(button,tube,fibstatus(chosen))
            call objover(nobj,object,otype,nsky,skypos,tube,button,
     :         fib(1,chosen),piv(1,chosen))
            go to 500
         end if
      end if
C
C     Ok... you've got the destination of the button, now check that
C     against all the other fibres to make sure there isn't a collision
C
      call do_fibre_check(npivots,piv1,but1,fibdims,chosen,piv,fib,
     :   fibrelen2,maxnonrad,fibstatus,offender,fstatus)
C
C     If there is a collision, then signal an error and return the fibre
C     to its proper colour.  Then get out of here...
C
      if (fstatus .ne. fchk_ok) then
         call mkmessage(chosen,offender,fstatus,message,lenm,status)
         call graphmsg(-1,'Cannot place fibre there, '//message(:lenm),
     :      2,pscale)
         call normfibre(button,tube,fibstatus(chosen))
         call objover(nobj,object,otype,nsky,skypos,tube,button,
     :      fib(1,chosen),piv(1,chosen))
         go to 500
      end if
C
C     Right, we've passed all the tests and we can procede with moving the 
C     fibre.  Start by erasing the old fibre position.
C
      call erase(button,tube,fibstatus(chosen))
      call objover(nobj,object,otype,nsky,skypos,tube,button,
     :   fib(1,chosen),piv(1,chosen))
C
C     If this fibre was on an object, then add that object back into the
C     reference lists
C
      call add_to_list(object_ref,n_obj_left,sky_ref,n_sky_left,
     :   allocated(chosen))
C
C     Increment the undo buffer and add the information on the old placement 
C
      undo_ptr = undo_ptr + 1
      if (undo_ptr .gt. maxundo) undo_ptr = undo_ptr - maxundo
      nundo = min(maxundo,nundo+1)
      nunun = max(0,nunun-1)
      undo_buf(2,undo_ptr) = allocated(chosen)
      undo_buf(3,undo_ptr) = fib(1,chosen)
      undo_buf(4,undo_ptr) = fib(2,chosen)
C
C     Assign the new button location
C
      fib(1,chosen) = nint(but1(1))
      fib(2,chosen) = nint(but1(2))
C
C     Now draw the fibre at its new position
C
      call calc_rects(piv1,but1,fibdims,button,tube)
      call normfibre(button,tube,fibstatus(chosen))
      call objover(nobj,object,otype,nsky,skypos,tube,button,
     :   fib(1,chosen),piv(1,chosen))
      call pgsci(1)
C
C     Now write an appropriate message and change the allocated status
C     of the fibre.
C
      len1 = 0
      if (dopark) then
         allocated(chosen) = 0
         call chr_putc('Parking fibre ',string,len1)
         call chr_puti(chosen,string,len1)
      else
         call chr_putc('Putting fibre ',string,len1)
         call chr_puti(chosen,string,len1)
         if (docurs) then
            call chr_putc(' on indicated location',string,len1)
            allocated(chosen) = alloc_cur
         else
            if (cur_obj .lt. alloc_sky) then
               call chr_putc(' on object ',string,len1)
               call chr_puti(oref,string,len1)
            else
               call chr_putc(' on sky position ',string,len1)
               call chr_puti(oref,string,len1)
            end if
            allocated(chosen) = cur_obj
         end if
      end if
      call graphmsg(-1,string(:len1),3,pscale)
C
C     Ok, we've placed the fibre on the object.  Now remove the object
C     from the reference lists
C
      call remove_from_list(object_ref,n_obj_left,sky_ref,n_sky_left,
     :   allocated(chosen))
C
C     Finally, put the new object information into the undo buffer.
C
      undo_buf(1,undo_ptr) = chosen
      undo_buf(5,undo_ptr) = allocated(chosen)
      undo_buf(6,undo_ptr) = fib(1,chosen)
      undo_buf(7,undo_ptr) = fib(2,chosen)
C
C     Exit
C
  500 continue
      end
