      subroutine newfield(fldfile,filename,npivots,otype,
     :   allocated,plog,modtar,status)
C+------------------------------------------------------------------------------
C
C     Name:   
C        newfield
C
C     Invocation:
C        call newfield(fldfile,filename,npivots,otype,
C           allocated,plog,modtar,status)
C
C     Description:
C        A new field file is created based on the current one.  The 
C        objects which have been allocated on this run are weighted
C        to zero.
C
C     Arguments: (">" Input, "<" Output, "W" Workspace, "!"  Changed)
C        fldfile:  Integer (>)
C           The input field file unit number.
C        filename:  Integer (>)
C           The name for the new field file (no extension)
C        npivots:  Integer (>)
C           The total number of pivots
C        otype:  Integer array (>)
C           The object type (i.e. whether it's a program or sky)
C        allocated:  Integer array (>)
C           The reference number of the object which has been allocated
C           to a particular fibre.
C        plog:  Character array (>)
C           The log of information on each of the objects
C        modtar:  Logical (>)
C           TRUE if you wish to modify the TARGET name in the output
C           field file.
C        status:  Integer (!)
C           The global status variable
C
C     Language:
C        FORTRAN 77
C
C     Non-standard language extensions:
C        include, implicit none, do while, variable names > 6 characters
C
C     Logical i/o units used:
C        41
C
C     External routines called:
C        Library chr:
C           chr_len
C 
C     Algorithm:
C        -- Open a new field file and rewind the old one.  Copy the
C           header information from the old to the new.
C        -- Loop through the pivots and find out which program objects
C           (not sky and not fiducial) have been allocated and store that
C           in a list.
C        -- Read each line of the input field file.  Break the line into
C           words.  If the object type is not 'P' (program) then just 
C           the line as it is.  If the object is a program object,
C           then compare the first 7 words of the data string to the
C           first 7 words in the program object log for each allocated 
C           object. (The first 7 words are the object's name and it's ra 
C           and dec).  If you find a match change the 9th word of the string
C           to '0'.  If you don't find a match then leave the string alone.
C        -- Write the string to the output file.
C        -- Continue to do this until you run out of data in the old file.
C
C     Authors:
C        (JRL): James R. Lewis -- RGO, Cambridge
C
C     History:
C        01 Dec 1995 (JRL):
C           Original version
C        03 Sep 1996 (JRL):
C           Added parameter 'modtar'
C        18 Sep 1996 (JRL):
C           Removed superfluous parameter 'nobj'
C
C-------------------------------------------------------------------------------
      implicit none
C
C     Global parameters
C
      include 'AF2_PAR.INC'
      include 'AF2_FSTA.INC'
C
C     Parameters
C
      integer fldfile
      character filename*(*)
      integer npivots
      integer otype(*)
      integer allocated(npivots)
      character plog(*)*80
      logical modtar
      integer status
C
C     Local parameters
C
      integer fd
      parameter (fd=41)
      integer maxword
      parameter (maxword=9)
C
C     Local variables
C
      integer iostat,ih,imt,ideg,ima,nread,ncover,i,oref,compare,j1,j2
      integer cover(nfibres),wrds,startw(maxword),stopw(maxword),lstat
      character fldlabel*80,centre*30,bundle*10,string*80
      character word(maxword)*30,stringout*80
      logical cont
      real date,equinox,hour_ang,st,sa,sky_pa
C
C     Functions
C
      integer chr_len
C
C     Inherited status
C
      if (status .ne. af2_ok) go to 500
C
C     Open the new field file
C
      call opnewfile(filename,'fld',fd,.false.,status)
      if (status .ne. af2_ok) go to 500
C
C     Now rewind the input field file
C
      rewind(fldfile,iostat=iostat)
      if (iostat .ne. 0) then
         write(*,fmt='(a)') 'Error rewinding input field file'
         status = af2_err
         go to 500
      end if
C
C     Read the header of the input field file and transfer the results
C     to the new field file...
C
      call rdhead(fldfile,0,fd,modtar,fldlabel,date,equinox,hour_ang,
     :   ih,imt,st,ideg,ima,sa,centre,sky_pa,bundle,nread,status)
      if (status .ne. af2_ok) go to 500
C
C     Now go through the pivots and find out which objects have been
C     allocated.  Skip sky patches and fiducials.
C
      ncover = 0
      do 10 i = 1,npivots
         oref = allocated(i)
         if ((oref .gt. 0) .and. (oref .lt. alloc_sky) .and. 
     :      (otype(oref) .eq. fsta_ook)) then
            ncover = ncover + 1
            cover(ncover) = oref
         end if
  10  continue
C
C     Now read through the input field file to the end.
C
      read(fldfile,fmt='(a)',iostat=iostat) string
      if (iostat .ne. 0) then
         status = af2_err
         write(*,fmt='(a)') 'Input field file has no object data!'
         go to 500
      end if
      do while (iostat .eq. 0)
C
C        Break the input string into words. Set the string comparison
C        length so that you compare the object name and its coordinates
C
         call chr_dcwrd(string,maxword,wrds,startw,stopw,word,lstat)
         compare = stopw(7)
C
C        If this is a fiducial or a sky line, then just copy the data over
C
         if (word(8)(1:1) .ne. 'P') then
            stringout = string
C
C        If this is a program object, then look to see if it's been allocated
C        This is done by comparing the current string with the log of all
C        of the allocated objects.
C
         else 
            cont = .true.
            i = 1
            do while (cont)
               if (plog(cover(i))(:compare) .eq. string(:compare)) then      
                  cont = .false.
               else
                  i = i + 1
                  if (i .gt. ncover) cont = .false.
               end if
            end do
C
C           If the object didn't match then just copy the string over
C
            if (i .gt. ncover) then
               stringout = string
C
C           If it did match, then change the value of the weight to zero
C           and then copy the string over...
C
            else
               j1 = startw(9)
               j2 = stopw(9)
               string(j1:j1) = '0'
               do 20 i = j1+1,j2
                  string(i:i) = ' '
  20           continue
               stringout = string
            end if
         end if
C
C        Now write this string to the output file
C
         write(fd,fmt='(a)') stringout(:chr_len(stringout))
C
C        Read the next line of the input file
C
         read(fldfile,fmt='(a)',iostat=iostat) string
      end do
C
C     Close up the new field file
C
      close(fd)
C
C     Exit
C
  500 continue
      end
