      subroutine place2(npivots,piv,fib,fibstatus,allocated,alloff,
     :   n_obj_left,object,weight,plog,otype,object_ref,maxnonrad,
     :   fibdims,fibrelen2,verbose,nnew,poss,nposs,hits,status)
C+------------------------------------------------------------------------------
C     Name:   
C        place2
C
C     Invocation:
C        call place2(npivots,piv,fib,fibstatus,allocated,alloff,
C           n_obj_left,object,weight,plog,otype,object_ref,maxnonrad,
C           fibdims,fibrelen2,verbose,nnew,poss,nposs,hits,status)
C
C     Description:
C        Fibres are placed on objects in a way which is designed to
C        maximise the total sum of weights of the objects which have
C        been allocated.
C
C     Arguments: (">" Input, "<" Output, "W" Workspace, "!"  Changed)
C        npivots:  Integer (>)
C           The total number of pivots
C        piv:  Integer array (>)
C           The x,y position of each of the pivots
C        fib:  Integer array (!)
C           The current x,y position of each fibre button
C        fibstatus:  Integer array (>)
C           The fibre status (i.e. whether it's useable, disabled etc.)
C        allocated:  Integer array (>)
C           The reference number of the object which has been allocated
C           to a particular fibre.
C        alloff:  Integer (>)
C           An offset used in the allocated array.  This is generally
C           used to distinguish a run where the objects are sky patches
C           from one where the objects are programs+fiducials.
C        n_obj_left:  Integer (!)
C           The number of objects which have not yet been allocated.
C        object:  Real array (>)
C           The x,y coordinates of the objects (program+fiducial)
C        weight:  Real array (>)
C           The weight assigned to each object
C        plog:  Character array (>)
C           The log of information on each of the objects
C        otype:  Integer array (>)
C           The object type (i.e. whether it's a program or sky)
C        object_ref:  Integer array (!)
C           The list of reference numbers of objects which have not 
C           been allocated to a fibre.
C        maxnonrad: Real (>)
C           The maximum non-radial angle each fibre is allowed to subtend
C        fibdims:  Integer array (>)
C           The dimensions of the fibres: 1 -- button length, 2 -- button
C           width, 3 -- tube width
C        fibrelen2:  Real (>)
C           The square of length of a fibre
C        verbose:  Logical (>)
C           TRUE if extra output is desired.
C        nnew:  Integer (<)
C           The number of new placements done on this run.
C        poss:  Real array (W)
C           Workspace.  Must have at least 3*npivots*n_obj_left elements
C        nposs:  Integer array (W)
C           Workspace.  Must have at least n_obj_left elements
C        hits:  Integer array (W)
C           Workspace.  Must have at least n_obj_left elements
C        status:  Integer (!)
C           The global status variable
C
C     Language:
C        FORTRAN 77
C
C     Non-standard language extensions:
C        include, implicit none, do while, variable names > 6 characters
C
C     Logical i/o units used:
C        none
C
C     External routines called:
C        Library subaf2_configure:
C           do_fibre_check
C           fibre_check
C           ssort
C 
C        Library chr:
C           chr_dcwrd
C           chr_len
C 
C     Algorithm:
C        -- First loop through the pivots and the object list and find all
C           of the possible fibre-object combinations which don't collide
C           with already placed fibres and which don't use zero weighted 
C           objects.
C        -- Next loop through the combination list.  See how many combinations
C           would be ruined by the current one.  Calculate a 'badness' of
C           the current combination by summing up the fraction of possible
C           combinations runined for a given object times that object's weight.
C           Sum this up over all objects and then divide by the weight of the
C           object involved in the currrent combination.
C
C           i.e. for the Kth combination involving the jth object, then:
C
C           badness(K) = SUM(i=1,nobj){weight(i)*nruined(i)/npossible(i)}
C                         where i != j
C           badness(K) = badness(K)/weight(j)
C
C        -- Sort the combinations by badness
C        -- Now loop through the combinations in order of increasing badness 
C           and place the best combinations.
C        -- Remove placed objects from the object reference list.
C
C     Authors:
C        (JRL): James R. Lewis -- RGO, Cambridge
C
C     History:
C        01 Dec 1995 (JRL):
C           Original version
C
C-------------------------------------------------------------------------------
      implicit none
C
C     Global parameters
C 
      include 'AF2_PAR.INC'
      include 'AF2_FCHK.INC'
C
C     Parameters
C
      integer npivots
      integer piv(2,npivots)
      integer fib(2,npivots)
      integer fibstatus(npivots)
      integer allocated(npivots)
      integer alloff
      integer n_obj_left
      real object(2,*)
      real weight(*)
      character plog(*)*80
      integer otype(*)
      integer object_ref(*)
      real maxnonrad
      integer fibdims(3)
      real fibrelen2
      logical verbose
      integer nnew
      real poss(3,*)
      integer nposs(*)
      integer hits(*)
      integer status
C
C     Local parameters
C
      real small
      parameter (small=1.0e-7)
      integer maxword
      parameter (maxword=1)
C
C     Local variables
C
      integer i,ncomb,j,oref,fstatus,offender,cur_obj,cur_fib,k
      integer cur_obj2,cur_fib2,nwrds,startw(maxword),stopw(maxword)
      integer lstat,len1,k2
      real piv1(2),but1(2),piv2(2)
      real but2(2),badness
      character word(maxword)*40
C
C     Functions
C
      integer chr_len
C
C     Inherited status
C
      if (status .ne. af2_ok) go to 500
C
C     Zero the number of new placements
C
      nnew = 0
C
C     If there aren't any more objects left in the list, then escape
C
      if (n_obj_left .eq. 0) go to 500
C
C     Initialise the counter which will record the number of good 
C     object-pivot combinations
C
      ncomb = 0
C
C     Message to keep the user interested...
C
      if (verbose) write(*,fmt='(a)') 
     :   'Looking for all possible object-fibre combinations...'
C
C     Go through all the objects and see which fibres can reach them.
C     Store all good object-fibre combinations.  Ignore all objects
C     with a zero weight...
C
      do 20 j = 1,n_obj_left
         oref = object_ref(j)
         nposs(j) = 0
         if (weight(oref) .gt. small) then
C
C           Loop for each of the pivots now.  If the current pivot isn't
C           already allocated and if the pivot has the right type of fibre
C           of the object then continue
C
            do 10 i = 1,npivots
               if ((fibstatus(i) .eq. otype(oref)) .and.
     :            (allocated(i) .eq. 0)) then
                  piv1(1) = real(piv(1,i))
                  piv1(2) = real(piv(2,i))
                  but1(1) = object(1,oref)
                  but1(2) = object(2,oref)
                  call do_fibre_check(npivots,piv1,but1,fibdims,i,
     :               piv,fib,fibrelen2,maxnonrad,fibstatus,offender,
     :               fstatus)
C
C                 If there aren't any collisions, then accept this 
C                 combination.  Increment the combination counter 
C                 and store the fibre and object numbers. Also
C                 increment the number of possible combinations
C                 for this particular object
C
                  if (fstatus .eq. fchk_ok) then
                     ncomb = ncomb + 1
                     poss(1,ncomb) = real(j)
                     poss(2,ncomb) = real(i)
                     poss(3,ncomb) = 0.0
                     nposs(j) = nposs(j) + 1
                  end if
               end if
C
C           End of pivot loop
C
  10        continue
         end if
C
C     End of object loop
C
  20  continue
C
C     Another message to keep the user interested...
C
      if (verbose) write(*,'(i6,a)') ncomb,
     :   ' possible object-fibre combinations found'
C
C     Check that we're not in the situation of having absolutely no
C     possible object-pivot combinations.  This may well be due to the
C     fact that most of the fibres or objects have already been allocated
C     so don't make this an error.
C
      if (ncomb .eq. 0) go to 500
C
C     Now we want do have a measure of the desirability of each fibre-pivot
C     combination.  This is done by measuring a 'badness' for each combination.
C     The badness is a measure of the number of combinations a particular
C     object-pivot combination would block and the weight of the blocked 
C     objects.  So start by looping for each combination.
C
      if (verbose) write(*,fmt='(a)') 
     :   'Now calculating badness index for each combination'
      do 60 i = 1,ncomb
         k = nint(poss(1,i))
         cur_obj = object_ref(k)
         cur_fib = nint(poss(2,i))
         piv1(1) = real(piv(1,cur_fib))
         piv1(2) = real(piv(2,cur_fib))
         but1(1) = object(1,cur_obj)
         but1(2) = object(2,cur_obj)
C
C        Zero the array which counts the number of collisions for a particular
C        object...
C
         do 30 j = 1,n_obj_left
            hits(j) = 0
  30     continue
C
C        Loop through the possibilities again, but skip current combination
C        and combinations which include the same object.
C
         do 40 j = 1,ncomb
            k2 = nint(poss(1,j))
            cur_obj2 = object_ref(k2)
            cur_fib2 = nint(poss(2,j))
            if ((cur_obj2 .ne. cur_obj) .and. (j .ne. i)) then
               piv2(1) = real(piv(1,cur_fib2))
               piv2(2) = real(piv(2,cur_fib2))
               but2(1) = object(1,cur_obj2)
               but2(2) = object(2,cur_obj2)
C
C              Now does the first combination cause a clash with the second?
C
               call fibre_check(piv1,but1,piv2,but2,fibdims,fstatus)
C
C              If they do collide then increment the number of hits on
C              this particular object...
C
               if (fstatus .ne. fchk_ok) hits(k2) = hits(k2) + 1
            end if
  40     continue
C
C        Now loop through the hits array and find out how many collisions there
C        were.  The badness index is incremented by an amount which reflects
C        the fraction of possible combinations removed for a particular
C        object and its weight.
C
         badness = 0.0
         do 50 j = 1,n_obj_left
            cur_obj2 = object_ref(j)
            if (nposs(j) .gt. 0) then
               badness = badness + 
     :            weight(cur_obj2)*real(hits(j))/real(nposs(j))
            end if
  50     continue
C
C        Scale the badness by the reciprocal of the weight of cur_obj
C        This is to take into account the fact that cur_obj may have a higher
C        weight than the objects it blocks.
C
         badness = badness/weight(cur_obj)
         poss(3,i) = badness
  60  continue
C
C     Now sort the combination array into increasing badness and cosort
C     the other columns
C
      if (verbose) write(*,fmt='(a)') 'Sorting by badness'
      call ssort(3,ncomb,poss,3)
C
C     Zero the hits array to use for keeping track of which objects have 
C     been covered
C
      do 70 i = 1,n_obj_left
         hits(i) = 0
  70  continue
C
C     Now loop for the sorted combinations and pick off all of the best
C     ones.  Keep track of which have been used to make sure you don't
C     try and cover the same object with two different fibres...
C
      if (verbose) write(*,fmt='(a)') 'Placing the fibres now...'
      do 80 i = 1,ncomb
         k = nint(poss(1,i))
         cur_obj = object_ref(k)
         cur_fib = nint(poss(2,i))
         if ((hits(k) .eq. 0) .and. 
     :      (allocated(cur_fib) .eq. 0)) then
C
C           Make sure that this doesn't collide with any of the 
C           already placed fibres.
C
            piv1(1) = real(piv(1,cur_fib))
            piv1(2) = real(piv(2,cur_fib))
            but1(1) = object(1,cur_obj)
            but1(2) = object(2,cur_obj)
            call do_fibre_check(npivots,piv1,but1,fibdims,cur_fib,
     :         piv,fib,fibrelen2,maxnonrad,fibstatus,offender,fstatus)
C
C           If there isn't a collision, then place that fibre.
C           Send out a useful message to the user.  Mark both
C           the fibre and the object as allocated and increment the
C           weight sum.
C
            if (fstatus .eq. fchk_ok) then
               fib(1,cur_fib) = nint(object(1,cur_obj))
               fib(2,cur_fib) = nint(object(2,cur_obj))
               if (verbose) then
                  call chr_dcwrd(plog(cur_obj),maxword,nwrds,startw,
     :               stopw,word,lstat)
                  len1 = chr_len(word(1))
                  write(*,fmt='(a,i3,a,i3,a,a)') 
     :               'putting fibre ',cur_fib,' on object ',cur_obj,
     :               ' ',word(1)(:len1)
               end if
               allocated(cur_fib) = cur_obj + alloff
               hits(k) = 1
               nnew = nnew + 1
            end if
         end if
  80  continue
C
C     Right, now we've allocated all of the objects we can.  
C     Shift all the uncovered objects up to the front of the list
C
      i = 0
      do 90 j = 1,n_obj_left
         if (hits(j) .eq. 0) then
            i = i + 1
            object_ref(i) = object_ref(j)
         end if
  90  continue
C
C     Decrease the number of objects left by the number covered on this run
C
      n_obj_left = n_obj_left - nnew
C
C     Exit
C
  500 continue
      end
