      subroutine placefibs(npivots,piv,fib,fibstatus,allocated,alloff,
     :   verbose,n_obj_left,object,weight,plog,otype,object_ref,
     :   maxnonrad,fibdims,fibrelen2,alg,useweight,nall,status)
C+------------------------------------------------------------------------------
C
C     Name:   
C        placefibs
C
C     Invocation:
C        call placefibs(npivots,piv,fib,fibstatus,allocated,alloff,
C           verbose,n_obj_left,object,weight,plog,otype,object_ref,
C           maxnonrad,fibdims,fibrelen2,alg,useweight,nall,status)
C
C     Description:
C        Allocate workspace and call the appropriate placement routine.
C
C     Arguments: (">" Input, "<" Output, "W" Workspace, "!"  Changed)
C        npivots:  Integer (>)
C           The total number of pivots
C        piv:  Integer array (>)
C           The x,y position of each of the pivots
C        fib:  Integer array (!)
C           The current x,y position of each fibre button
C        fibstatus:  Integer array (>)
C           The fibre status (i.e. whether it's useable, disabled etc.)
C        allocated:  Integer array (>)
C           The reference number of the object which has been allocated
C           to a particular fibre.
C        alloff:  Integer (>)
C           An offset used in the allocated array.  This is generally
C           used to distinguish a run where the objects are sky patches
C           from one where the objects are programs+fiducials.
C        verbose:  Logical (>)
C           TRUE if extra output is desired.
C        n_obj_left:  Integer (!)
C           The number of objects which have not yet been allocated.
C        object:  Real array (>)
C           The x,y coordinates of the objects (program+fiducial)
C        weight:  Real array (>)
C           The weight assigned to each object
C        plog:  Character array (>)
C           The log of information on each of the objects
C        otype:  Integer array (>)
C           The object type (i.e. whether it's a program or sky)
C        object_ref:  Integer array (!)
C           The list of reference numbers of objects which have not 
C           been allocated to a fibre.
C        maxnonrad: Real (>)
C           The maximum non-radial angle each fibre is allowed to subtend
C        fibdims:  Integer array (>)
C           The dimensions of the fibres: 1 -- button length, 2 -- button
C           width, 3 -- tube width
C        fibrelen2:  Real (>)
C           The square of length of a fibre
C        alg:  Integer (>)
C           The number of the algorithm to be used. 1 -- FAST algorithm,
C           2 -- SLOW algorithm
C        useweight:  Logical (>)
C           TRUE if you want to exclude objects which are zero weighted.
C        nall:  Integer (<)
C           The number of new placements done on this run.
C        status:  Integer (!)
C           The global status variable
C
C     Language:
C        FORTRAN 77
C
C     Non-standard language extensions:
C        include, implicit none, do while, %val, variable names > 6 characters
C
C     Logical i/o units used:
C        none
C
C     External routines called:
C        Library subaf2_configure:
C           csub_free
C           csub_malloc
C           csub_typesize
C           place1
C           place2
C 
C     Algorithm:
C        -- Allocate workspace according to the amount needed for the 
C           chosen algorithm.
C        -- Call the chosen algorithm
C        -- Free the workspace
C
C     Authors:
C        (JRL): James R. Lewis -- RGO, Cambridge
C
C     History:
C        01 Dec 1995 (JRL):
C           Original version
C        03 Sep 1996 (JRL):
C           Replaced logical parameter 'newalg' with integer parameter 'alg'
C           in order to allow for more placement algorithms in the future.
C
C-------------------------------------------------------------------------------
      implicit none
C
C     Global parameters
C
      include 'AF2_PAR.INC'
C
C     Parameters
C
      integer npivots
      integer piv(2,npivots)
      integer fib(2,npivots)
      integer fibstatus(npivots)
      integer allocated(npivots)
      integer alloff
      logical verbose
      integer n_obj_left
      real object(2,*)
      real weight(*)
      character plog(*)*80
      integer otype(*)
      integer object_ref(*)
      real maxnonrad
      integer fibdims(3)
      real fibrelen2
      integer alg
      logical useweight
      integer nall
      integer status
C
C     Local variables
C
      integer ntot,szwork,work1,work2,work3,nnew,nbi,nbr
      logical cont
C
C     Functions
C
      integer csub_typesize
C
C     Inherited status
C
      if (status .ne. af2_ok) go to 500
C
C     Get the number of bytes in the integer and real data types
C
      nbi = csub_typesize(ty_int,status)
      nbr = csub_typesize(ty_float,status)
      if (status .ne. af2_ok) go to 500
C
C     Get the workspace necessary for each of the placement algorithms.
C     This is expressed in the number of bytes needed.
C
      ntot = max(1,n_obj_left)
      if (alg .eq. 2) then
         szwork = ntot*(3*npivots*nbr + 2*nbi)
      else
         szwork = 2*ntot*nbr
      end if
      call csub_malloc(szwork,work1,status)
      if (status .ne. af2_ok) go to 500
C
C     The SLOW algorithm requires three workspaces.  Increment the
C     first workspace pointer for the second two workspaces.
C
      if (alg .eq. 2) then
         work2 = work1 + ntot*npivots*3*nbr
         work3 = work2 + ntot*nbi
      end if
C
C     Ok...Now if you are using the SLOW algorithm then call it here
C     for the object+fiducial data.
C
      if (alg .eq. 2) then
         nall = 0
         cont = .true.
         do while (cont)
            call place2(npivots,piv,fib,fibstatus,allocated,alloff,
     :         n_obj_left,object,weight,plog,otype,object_ref,maxnonrad,
     :         fibdims,fibrelen2,verbose,nnew,%val(work1),%val(work2),
     :         %val(work3),status)
            if (status .ne. af2_ok) go to 500
            nall = nall + nnew
            cont = nnew .gt. 0
         end do
C
C     Otherwise, if you are using the old algorithm, then keep calling
C     it until you stop allocating objects.
C
      else
         nall = 0
         cont = .true.
         do while (cont)
            call place1(npivots,piv,fib,fibstatus,allocated,alloff,
     :         n_obj_left,object,weight,plog,otype,object_ref,
     :         maxnonrad,fibdims,fibrelen2,verbose,nnew,useweight,
     :         %val(work1),status)
            if (status .ne. af2_ok) go to 500
            nall = nall + nnew
            cont = nnew .gt. 0
         end do
      end if
C
C     Free the workspace
C
      call csub_free(work1,status)
C
C     Exit
C
  500 continue
      end
