      subroutine restore(n_obj_left,object_ref,n_sky_left,sky_ref,
     :   npivots,fib,allocated,mode,status)
C+------------------------------------------------------------------------------
C
C     Name:   
C        restore
C
C     Invocation:
C        call restore(n_obj_left,object_ref,n_sky_left,sky_ref,
C           npivots,fib,allocated,mode,status)
C
C     Description:
C        Allocate memory to hold a configuration and copy a configuration
C        to it.  If requested, replace the current configuration with that
C        which is stored.
C
C     Arguments: (">" Input, "<" Output, "W" Workspace, "!"  Changed)
C        n_obj_left:  Integer (!)
C           The number of objects which have not yet been allocated.
C        object_ref:  Integer array (!)
C           The list of reference numbers of objects which have not 
C           been allocated to a fibre.
C        n_sky_left:  Integer (!)
C           The number of sky patches which have not yet been allocated
C        sky_ref:  Integer array (!)
C           The list of reference numbers of the sky patches which have
C           not been allocated to a fibre
C        npivots:  Integer (>)
C           The total number of pivots
C        fib:  Integer array (!)
C           The current x,y position of each fibre button
C        allocated:  Integer array (!)
C           The reference number of the object which has been allocated
C           to a particular fibre.
C        mode:  Integer (>)
C           The value which determines what this routine will do:  
C           1 -- allocate the workspace and copy the current configuration 
C                arrays to it
C           2 -- copy the configuration arrays held in the workspace into
C                the current configuration arrays.
C           3 -- free the workspace
C        status:  Integer (!)
C           The global status variable
C
C     Language:
C        FORTRAN 77
C
C     Non-standard language extensions:
C        include, implicit none, %val, variable names > 6 characters
C
C     Logical i/o units used:
C        none
C
C     External routines called:
C        Library subaf2_configure:
C           csub_free
C           csub_malloc
C           csub_typesize
C           trans
C 
C     Algorithm:
C        -- If mode = 1, get the number of bytes for integer type and
C           allocate the total workspace required.  Break the workspace up
C           into separate arrays and copy the original configuration to
C           them.
C        -- If mode = 2, then copy the configuration held in the workspace
C           into the input configuration arrays, thus overwriting the 
C           current configuration.
C        -- If mode = 3, then free the workspace.
C
C     Authors:
C        (JRL): James R. Lewis -- RGO, Cambridge
C
C     History:
C        01 Dec 1995 (JRL):
C           Original version
C        31 Jan 1996 (JRL):
C           Fixed wrong number of arguments to csub_free
C
C-------------------------------------------------------------------------------
      implicit none
C
C     Global parameters
C
      include 'AF2_PAR.INC'
C
C     Parameters
C
      integer n_obj_left
      integer object_ref(*)
      integer n_sky_left
      integer sky_ref(*)
      integer npivots
      integer fib(2,npivots)
      integer allocated(npivots)
      integer mode
      integer status
C
C     Local variables
C
      integer nbi,totbytes,ptr,wknobjleft,wknskyleft,wkobjref,wkskyref
      integer wkfib,wkalloc
C
C     Save the pointer values from one call to the next...
C
      save ptr,wknobjleft,wknskyleft,wkobjref,wkskyref,wkfib,wkalloc
C
C     Functions
C
      integer csub_typesize
C
C     Inherited status
C
      if (status .ne. af2_ok) go to 500
C
C     If the mode of operation variable is 1 then allocate the necessary
C     workspace and copy the information into it.
C
      if (mode .eq. 1) then
C
C        First, how much space do we need?  Get the number of bytes for
C        the integer type.  Then calculate the amount of workspace.
C        Allocate the workspace
C
         nbi = csub_typesize(TY_INT,status)
         totbytes = (n_obj_left + n_sky_left + 3*npivots + 2)*nbi
         call csub_malloc(totbytes,ptr,status)
         if (status .ne. af2_ok) go to 500
C
C        Now divide up the workspace for each array.
C
         wknobjleft = ptr
         wknskyleft = wknobjleft + nbi
         wkobjref = wknskyleft + nbi
         wkskyref = wkobjref + n_obj_left*nbi
         wkfib = wkskyref + n_sky_left*nbi
         wkalloc = wkfib + 2*npivots*nbi
C
C        Now transfer the data over
C
         call trans(1,n_obj_left,%val(wknobjleft))
         call trans(1,n_sky_left,%val(wknskyleft))
         call trans(n_obj_left,object_ref,%val(wkobjref))
         call trans(n_sky_left,sky_ref,%val(wkskyref))
         call trans(2*npivots,fib,%val(wkfib))
         call trans(npivots,allocated,%val(wkalloc))
C
C     If the mode variable is 2, then do just the opposite -- transfer the
C     old data held in memory. Don't free the memory in case you need to
C     do this operation again!
C
      else if (mode .eq. 2) then
         call trans(1,%val(wknobjleft),n_obj_left)
         call trans(1,%val(wknskyleft),n_sky_left)
         call trans(n_obj_left,%val(wkobjref),object_ref)
         call trans(n_sky_left,%val(wkskyref),sky_ref)
         call trans(2*npivots,%val(wkfib),fib)
         call trans(npivots,%val(wkalloc),allocated)
         if (status .ne. af2_ok) go to 500
C
C     If the mode variable is 3 then just free the workspace
C
      else if (mode .eq. 3) then
         call csub_free(ptr,status)
      end if
C
C     Exit
C
  500 continue
      end

      subroutine trans(nthings,arrayin,arrayout)
C+------------------------------------------------------------------------------
C
C     Name:   
C        trans
C
C     Invocation:
C        call trans(nthings,arrayin,arrayout)
C
C     Description:
C        Transfer the elements from one array to another
C
C     Arguments: (">" Input, "<" Output, "W" Workspace, "!"  Changed)
C        nthings:  Integer (>)
C           The total number of elements in the input array
C        arrayin:  Integer array (>)
C           The input array
C        arrayout:  Integer array (<)
C           The output array
C
C     Language:
C        FORTRAN 77
C
C     Non-standard language extensions:
C        implicit none, variable names > 6 characters
C
C     Logical i/o units used:
C        none
C
C     External routines called:
C        none
C 
C     Algorithm:
C        -- Copy all the elements of one array into another
C
C     Authors:
C        (JRL): James R. Lewis -- RGO, Cambridge
C
C     History:
C        01 Dec 1995 (JRL):
C           Original version
C
C-------------------------------------------------------------------------------
      implicit none
C
C     Parameters
C
      integer nthings
      integer arrayin(nthings)
      integer arrayout(nthings)
C
C     Local variables
C
      integer i
C
C     Transfer the data across
C
      do 10 i = 1,nthings
         arrayout(i) = arrayin(i)
  10  continue
      end          
